<?php
/**
 * @category     Addons
 * @package      page module: faqbaker
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com>
 * @author       Uwe Jacobsen <webdesign@jacobi22.com>
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      2.2.0
 * @lastmodified $Date: 2023/03/09
 * @since        File available since 2023/03/09
 * @description  addon faqbaker
 * class.Faqbaker.php
*/

namespace addon\faqbaker\Lib;

use bin\{WbAdaptor, SecureTokens};
use bin\helpers\{PreCheck,StopWatch};
use src\Security\{CsfrTokens, Randomizer, Password};
use Twig\Loader\FilesystemLoader;
use Twig\Environment;
use Twig\TwigFilter;

    class Faqbaker {

    /** @var object instance of the application object */
        protected $_oApp;
    /** @var object instance of the database object */
        protected $_oDb ;
    /** @var object instance holds several values from the application global scope */
        protected $_oReg;
    /** @var object instance holds all of the translations */
        protected $_oTrans;


    // -----------------------------------------------------------------------------
    /** prevent class from public instancing and get an object to hold extensions */
    public function __construct()
    {
        // import global vars and objects
        $this->_oReg   = WbAdaptor::getInstance();
        $this->_oDb    = $this->_oReg->getDatabase();
        $this->_oTrans = $this->_oReg->getTranslate();
        $this->_oApp   = $this->_oReg->getApplication();
    }
    public function getFaqbakerSettings($iSectionId) {
        return $this->_readSettings($iSectionId);
    }
    public function FAQCatEntrys($iSectionId) {
        return $this->_readFAQCatEntrys($iSectionId);
    }
    public function FAQCatSortMode($iSectionId) {
        return $this->_readFAQCatSortMode($iSectionId);
    }
    public function FAQQuestEntrys($iSectionId) {
        return $this->_readFAQQuestEntrys($iSectionId);
    }
    public function FAQQuestSortMode($iSectionId) {
        return $this->_readFAQQuestSortMode($iSectionId);
    }
    public function getModuleType($iSectionId) {
        return $this->_readModuleType($iSectionId);
    }
        ###############################################################################
        // read settings for this section (getFaqbakerSettings)
        ###############################################################################
        /** read default setting for this section */
                 protected function  _readSettings($iSectionId){
                    global $aDefaults, $oAddonReg, $section_id, $page_id, $sAddonPath ;
//                    $oDb = \database::getInstance();
                        $aFaqbakerSettings  = [];
                        $sql  = 'SELECT `s_name`, `s_value` FROM `'. $this->_oDb->TablePrefix .'mod_faq_settings` '
                              . 'WHERE `section_id` = ' . $iSectionId;
//                        echo $sql;
                        if ($query = $this->_oDb->query($sql)){
                            while ($row = $query->fetchRow(MYSQLI_ASSOC)) {
                                $aFaqbakerSettings[$row['s_name']] = $row['s_value'];
                            }
                        }

                        // insert default_settings if no settings for this section avaiable
                        if (sizeof($aFaqbakerSettings) == 0){
                            $aFieldsList = array(
                                'section_id' => $section_id,
                                'page_id'    => $page_id,
                            );
                            /* add default setting if not set for this section */
                            if (\is_array($aDefaults)) {
                                $aTmp = \array_diff_key($aDefaults, $aFaqbakerSettings);
                                if ($aTmp){
                                    UpdateKeyValue('mod_faq_settings', $aDefaults, '', $aFieldsList);
                                }
                            }
                        }
//                        $sql= 'SELECT COUNT(*) FROM `'.$this->_oDb->TablePrefix.'mod_faq_templates` '
//                             . 'WHERE `section_id` = ' . $iSectionId;
//                        $iCountTemplate = $this->_oDb->get_one($sql);
//                        if ($iCountTemplate == 0){
//                            
//                            $position      = 1;
//                            $active        = 1;
//                            if ($aFaqbakerSettings['use_glossar'] == 0){
//                                if (is_readable($sAddonPath.'/presets/default.template.inc.php')){require ($sAddonPath.'/presets/default.template.inc.php');}
//                                $sTemplateName = 'Default DIV-Template';
//                                $sTemplateDesc = 'Output in DIV-Structure, usable in FAQ-Module';
//                            }else{
//                                if (is_readable($sAddonPath.'/presets/glossar.template.inc.php')){require ($sAddonPath.'/presets/glossar.template.inc.php');}
//                                $sTemplateName = 'Default DIV-Template for Glossar';
//                                $sTemplateDesc = 'Output in DIV-Structure, usable in Glossar (use a Submenu für categories)';
//                            }
//                            $sql =  'INSERT INTO `'.$this->_oDb->TablePrefix.'mod_faq_templates` SET '
//                                        .  '`section_id`      = '.(int)$iSectionId.', '
//                                        .  '`template_name`   = \''.$this->_oDb->escapeString($sTemplateName).'\', '
//                                        .  '`template_desc`   = \''.$this->_oDb->escapeString($sTemplateDesc).'\', '
//                                        .  '`template_code`   = \''.$this->_oDb->escapeString($sTemplateCode).'\', '
//                                        .  '`position`        = '.(int)$position.', '
//                                        .  '`active`          = '.(int)$active.' ';
//                            $this->_oDb->query($sql);
////                            echo $sql;
//                            if($this->_oDb->is_error()) {
//                                $aMsg[] = $database->get_error().'<br>';
//                                $aMsg[] = '<strong>\'FAQ Baker Insert Template Preset \'</strong><br>';
//                            }else {
//                                $aMsg[] = '<strong>\'FAQ Baker recreate templates\'</strong><br>';
//                            }
//                        }
                       
//            echo "<pre>";
//            print_r($aFaqbakerSettings);
//            echo "</pre>";    
                return $aFaqbakerSettings;
                }
        // end of static public function ('getFaqbakerSettings')
        //

        ###############################################################################
        // count groups for this section (_readFAQCatEntrys)
        ###############################################################################
                protected function _readFAQCatEntrys($iSectionId){
            $sql= 'SELECT COUNT(*) FROM `'. $this->_oDb->TablePrefix .'mod_faq_categories` '
                . 'WHERE `section_id` = '.$iSectionId.' ';
            $iCountEntrys = $this->_oDb->get_one($sql);
//            echo "<pre>";
//            print_r($iCountEntrys);
//            echo "</pre>";
            return (@$this->_oDb->get_error()?:$iCountEntrys);
        }
        // end of function_exists('_readFAQCatEntrys')

        ###############################################################################
        // get sort order for groups this section (_readFAQCatSortMode)
        ###############################################################################
                protected function _readFAQCatSortMode($iSectionId){
                    $sql= 'SELECT `s_value` FROM `'. $this->_oDb->TablePrefix .'mod_faq_settings` '
                        . 'WHERE `section_id` = '.$iSectionId.' '
                        . 'AND `s_name` = \'sort_cat_name\' '    ;
                    $iCatSortMode = $this->_oDb->get_one($sql);
            
                    return (@$this->_oDb->get_error()?:$iCatSortMode);
        }
         // end of function_exists('_readFAQCatSortMode')
         //

        ###############################################################################
        // count Questions for this section (_readFAQQuestEntrys)
        ###############################################################################
            protected function _readFAQQuestEntrys($iSectionId){
                $sql= 'SELECT COUNT(*) FROM `'. $this->_oDb->TablePrefix .'mod_faq_questions` '
                    . 'WHERE `section_id` = '.$iSectionId.' ';
                $iCountEntrys = $this->_oDb->get_one($sql);
                return (@$this->_oDb->get_error()?:$iCountEntrys);
            }
         // end of function('_readFAQQuestEntrys')
         // 

        ###############################################################################
        // get sort order for Questions in Cats in this section (_readFAQQuestSortMode)
        ###############################################################################
        protected function _readFAQQuestSortMode($iSectionId){
            $sql= 'SELECT `s_value` FROM `'. $this->_oDb->TablePrefix .'mod_faq_settings` '
                . 'WHERE `section_id` = '.$iSectionId.' '
                .'AND `s_name` = \'sort_quest_in_cats\' ';
            $iQuestSortMode = $this->_oDb->get_one($sql);
            return (@$this->_oDb->get_error()?:$iQuestSortMode);
        }
         // end of function('_readFAQQuestSortMode') 
         // 

        ################################################################################
        // get module type for this section (glossary(1) or faq(0)) (_readModuleType)
        ###############################################################################
        protected function _readModuleType($iSectionId){
            $sql= 'SELECT `s_value` FROM `'. $this->_oDb->TablePrefix .'mod_faq_settings` '
                . 'WHERE `section_id` = '.$iSectionId.' '
                . 'AND `s_name` = \'use_glossar\' '    ;
            $sFeTemplateType = $this->_oDb->get_one($sql);
            return (@$this->_oDb->get_error()?:$sFeTemplateType);
        }
         // end of function_exists('_readModuleType')

        ################################################################################
        // get Frontend-template for this section (_readFAQFeTemplateID)
        ###############################################################################
        protected function _readFAQFeTemplateID($iSectionId){
            $sql= 'SELECT `s_value` FROM `'. $this->_oDb->TablePrefix .'mod_faq_settings` '
                . 'WHERE `section_id` = '.$iSectionId.' '
                . 'AND `s_name` = \'template\' '    ;
            $sFeTemplate = $this->_oDb->get_one($sql);
            return (@$this->_oDb->get_error()?:$sFeTemplate);
        }
         // end of function_exists('_readFAQFeTemplateID')
        
        ################################################################################
        // get Frontend-template-code for this section (getFAQFrontendTemplate)
        ###############################################################################
        static public function getFAQFrontendTemplate($iSectionId, $iTemplateId){
            global $database;
            $sql = 'SELECT `template_code` FROM `'.TABLE_PREFIX.'mod_faq_templates` '
                 . 'WHERE  `section_id` = '.$iSectionId.' '
                 . 'AND `template_id` = '.$iTemplateId.' '
                 . 'AND `active` = 1';
            $sTemplateCode = $database->get_one($sql);
            return (@$database->get_error()?:$sTemplateCode);
        }
         // end of function_exists('getFAQFrontendTemplate')
         //        
################################################################################
// get Frontend-template for this section (getFAQFeTemplateID)
###############################################################################
        static public function getFAQFeTemplateID($iSectionId)
            {
            global $database;
            $sql= 'SELECT `s_value` FROM `'.TABLE_PREFIX.'mod_faq_settings` '
                . 'WHERE `section_id` = '.$iSectionId.' '
                . 'AND `s_name` = \'output_template\' '    ;
            $sFeTemplate = $database->get_one($sql);
//            echo "<pre>Template";
//            print_r( $sFeTemplate);
//            echo "</pre>";
            return (@$database->get_error()?:$sFeTemplate);
        }
 // end of function_exists('getFAQFeTemplateID')
 //
###############################################################################
// get all cat names in a list for this section (getFAQAllCatTitles)
###############################################################################
        static public function getFAQAllCatTitles($iSectionId)
            {
                global $database,$wb;
                $aResult = []; 
                $sql = 'SELECT `c`.`cat_name` '
                     . 'FROM `'.TABLE_PREFIX.'mod_faq_questions` `q` '
                     . 'LEFT OUTER JOIN `'.TABLE_PREFIX.'mod_faq_categories` `c` ON `q`.`cat_id`=`c`.`cat_id` '
                     . 'WHERE `q`.`section_id`='.$iSectionId.' '
                     . ' AND `c`.`active`= 1 ' 
                     . ' AND `q`.`active`= 1 ';
                if (($oCats = $database->query($sql))) {
                    while($aCats = $oCats->fetchRow(MYSQLI_ASSOC)) {
                        $aResult[] = $aCats['cat_name'];
                        }
                    }
    //            echo "<pre>";
    //            print_r($aResult);
    //            echo "</pre>";
            // return the complete result structure
            return $aResult;
            }
// // end of function_exists('getFAQAllCatTitles')

 //  ###############################################################################
// get sort order for Questions in Cats in this section (getFAQQuestSortMode)
//        static public function getFAQQuestSortMode($iSectionId)
//            {
//                global $database;
//                $sql= 'SELECT `s_value` FROM `'.TABLE_PREFIX.'mod_faq_settings` '
//                    . 'WHERE `section_id` = '.$iSectionId.' '
//                    .'AND `s_name` = \'sort_quest_in_cats\' ';
//                $iQuestSortMode = $database->get_one($sql);
//                return (@$database->get_error()?:$iQuestSortMode);
//        }

 // end of function_exists('getFAQQuestSortMode') 
 //  
//         ################################################################################
//// get Frontend-template-code for this section (getFAQFrontendTemplate)
//###############################################################################
//        static public function getFAQFrontendTemplate($iSectionId, $iTemplateId)
//            {
//                global $database;
//                $sql = 'SELECT `template_code` FROM `'.TABLE_PREFIX.'mod_faq_templates` '
//                     . 'WHERE  `section_id` = '.$iSectionId.' '
//                     . 'AND `template_id` = '.$iTemplateId.' ';
//                $sTemplateCode = $database->get_one($sql);
//                return (@$database->get_error()?:$sTemplateCode);
//        }
// // end of function_exists('getFAQFrontendTemplate')
// //
        ###############################################################################
        // count different templates for this section (getFAQTemplateEntrys)
        ###############################################################################
        static public function getFAQTemplateEntrys($iSectionId){
            global $database;
            $sql= 'SELECT COUNT(*) FROM `'.TABLE_PREFIX.'mod_faq_templates` '
                      . 'WHERE `section_id` = '.$iSectionId.' ';
            $iCountTemplate = $database->get_one($sql);
            return (@$database->get_error()?:$iCountTemplate);
        }
         // end of function_exists('getFAQTemplateEntrys')
    
         ###############################################################################
        // count different templates for this section (getFAQALLTemplateEntrys)
        ###############################################################################
//        static public function getFAQALLTemplateEntrys(){
//            global $database;
//            $sql= 'SELECT COUNT(*) FROM `'.TABLE_PREFIX.'mod_faq_templates` ';
//            $iCountTemplate = $database->get_one($sql);
//            return (@$database->get_error()?:$iCountTemplate);
//        }
         // end of function_exists('getFAQTemplateEntrys')
         //
        ################################################################################
        // backend overview - complete List of all Templates (getFAQAllTemplatesArray)
        ###############################################################################
        static public function getFAQAllTemplatesArray($iSectionId){ 
            global $database, $admin;
            // create an empty result
            $aResult = [];    
            // build the SQL-Statement to select from booth relating tables
            $sql = 'SELECT * FROM `'.TABLE_PREFIX.'mod_faq_templates` '
                                 . 'WHERE  `section_id` = '.$iSectionId.' '
                                 . 'ORDER by `position`';
            if (($oGroups = $database->query($sql))) {
                        while($aGroups = $oGroups->fetchRow(MYSQLI_ASSOC)) {
                        $aGroup = array();
                        // items from current group
                        $aGroup['template_id']        = $aGroups['template_id'];
                        $aGroup['template_idkey']     = $admin->getIDKEY($aGroups['template_id']);
                        $aGroup['template_desc']      = $aGroups['template_desc'];
                        $aGroup['template_name']      = $aGroups['template_name'];
                        $aGroup['position']           = $aGroups['position'];
                        $aGroup['active']             = $aGroups['active'];
                        $aResult[] = $aGroup;
                        }
                    }
                #echo "<pre>";
                #print_r($aResult);
                #echo "</pre>";
            // return the complete result structure
            return $aResult;
        } 
        
        ###############################################################################
        // backend overview cats and questionslist (getFAQCatQuestArray)
        ###############################################################################
        static public function getFAQCatQuestArray($iSectionId,$iCatOrder,$iQuestOrder){
            // import the global admin nand database objects
                global $database, $admin;
            // create an empty result array
                $aResult = [];
                $now = time()+ TIMEZONE; 
            // SQL-Statement to all cats of a section; ordered by cat_id
                if($iCatOrder == 1){ 
                        $iCategoryOrder = '`position` ASC';
                    }elseif($iCatOrder == 2){ 
                        $iCategoryOrder = '`position` DESC';
                    }elseif($iCatOrder == 3){ 
                        $iCategoryOrder = '`cat_name` ASC';
                    }elseif($iCatOrder == 4){ 
                        $iCategoryOrder = '`cat_name` DESC';
                    }else{ 
                        $iCategoryOrder = '`position` ASC';
                    }
                    if($iQuestOrder == 1){ 
                        $iQuestionOrder = '`position` ASC';
                    }elseif($iQuestOrder == 2){ 
                        $iQuestionOrder = '`position` DESC';
                    }elseif($iQuestOrder == 3){ 
                        $iQuestionOrder = '`question` ASC';
                    }elseif($iQuestOrder == 4){ 
                        $iQuestionOrder = '`question` DESC';
                    }else{ 
                        $iQuestionOrder = '`position` ASC';
                    }
//                    echo $iQuestOrder."Questorder = ".$iQuestionOrder."<br>";
                $sql = 'SELECT `cat_id`, `cat_name`, `cat_desc`, '
                     . '`position` AS `cat_position`, `active` AS `cat_active`, `section_id` AS `cat_section` '
                     . 'FROM `'.TABLE_PREFIX.'mod_faq_categories` '
                     . 'WHERE `section_id` = '.(int)$iSectionId.' '
                     . 'ORDER BY '.$iCategoryOrder.'  ';
//                        echo $sql."<br>";
                if (($oCats = $database->query($sql))) {
                // iterate all cats of one section
                    while (($aCat = $oCats->fetchRow(MYSQLI_ASSOC))) {
                        $aQuestions = []; // empty files array of this cat
                        $iQuestCounter = 0;
                        $iQuestCounter = $database->get_one('SELECT COUNT(*) FROM `'.TABLE_PREFIX.'mod_faq_questions` WHERE `section_id` = '.$iSectionId.' AND `cat_id` = '.$aCat['cat_id'].' ');
                        $aCat['questCounter'] = $iQuestCounter;
        //                $aCat['cat_position'] = $aCat['cat_position'];
                        $aCat['cat_idkey'] = $admin->getIDKEY($aCat['cat_id']);
                        $aCat['countCats']  = 0;
                        $sql = 'SELECT `question_id`, `question`, `position` AS `quest_position`,`cat_id` AS `quest_cat_id`, '
                        . '`answer` AS `quest_answer`, `active` AS `quest_active`, `section_id` AS `quest_section`, '
                        . '`modified_when`, `modified_by`, `published_when`, `published_until`,`page_id` '
                        . 'FROM `'.TABLE_PREFIX.'mod_faq_questions` '
                        . 'WHERE `cat_id`='.(int)$aCat['cat_id'].' '
                        . 'AND `section_id`='.(int)$iSectionId.' '
                        . 'ORDER BY '.$iQuestionOrder.' ';
//                        echo $sql."<br>";
                        // get the files of this cat
                        if (($oItems = $database->query($sql))) {
                        // iterate all files of a cat
                            while($aItem = $oItems->fetchRow(MYSQLI_ASSOC)) {
                                $sCatDesc = OutputFilterApi('ReplaceSysvar', $aCat['cat_desc']);
                                $sCatDesc = htmlspecialchars_decode($sCatDesc);
                                //
                                $sQuestAnswer = OutputFilterApi('ReplaceSysvar', $aItem['quest_answer']);
                                $sQuestAnswer = strip_tags($sQuestAnswer);
                                $sql = 'SELECT `link` FROM `'.TABLE_PREFIX.'pages` '
                                      .'WHERE `page_id` = '.$aItem['page_id'].' ';
                                $sPageLink = $database->get_one($sql);
                                $sPermalink = WB_URL.PAGES_DIRECTORY.$sPageLink.PAGE_EXTENSION.'#quest'.$aItem['question_id'];
        //                        
                                ++$aCat['countCats'];
                                // items from current category
                                $aCurrItem['cat_name']         = $aCat['cat_name'];
                                $aCurrItem['cat_desc']         = $aCat['cat_desc'];
                                $aCurrItem['cat_active']       = $aCat['cat_active'];
                                $aCurrItem['cat_position']     = $aCat['cat_position'];
                                $aCurrItem['cat_section']      = $aCat['cat_section'];
                                $aCurrItem['quest_cat_id']     = $aItem['quest_cat_id'];
                                $aCurrItem['quest_cat_idkey']  = $admin->getIDKEY($aItem['quest_cat_id']);
                                $aCurrItem['question_id']      = $aItem['question_id'];
                                $aCurrItem['question_idkey']   = $admin->getIDKEY($aItem['question_id']);
                                $aCurrItem['quest_answer']     = $sQuestAnswer;
                                $aCurrItem['question']         = $aItem['question'];
                                $aCurrItem['quest_position']   = $aItem['quest_position'];
                                $aCurrItem['quest_active']     = $aItem['quest_active'];
                                $aCurrItem['quest_section']    = $aItem['quest_section'];
                                $aCurrItem['modified_when']    = $aItem['modified_when'];
                                $aCurrItem['modified_by']      = $aItem['modified_by'];
                                $aCurrItem['published_when']   = $aItem['published_when'];
                                $aCurrItem['published_until']  = $aItem['published_until'];
                                $aCurrItem['permalink']        = $sPermalink;
                                $aQuestions[] = $aCurrItem;
                            }
                        }
                        // add items to the category or an empty array if there no items
                        $aCat['questions'] = $aQuestions;
                        // add category to the result
                        $aResult[] = $aCat;
                    }
                }
        //            echo "<pre>";
        //            print_r($aResult);
        //            echo "</pre>";
                // return the complete result structure
                return $aResult;
            } // end of function

        ###############################################################################
        // backend overview - complete List of all Cats (getFAQAllCatsArray)
        ###############################################################################
        static public function getFAQAllCatsArray($iSectionId){ 
            global $database, $admin;
            // create an empty result
            $aResult = [];    
            // build the SQL-Statement to select from booth relating tables
            $sql = 'SELECT * FROM `'.TABLE_PREFIX.'mod_faq_categories` '
                                 . 'WHERE  `section_id` = '.$iSectionId.' '
                                 . 'ORDER by `position`';
            if (($oCats = $database->query($sql))) {
                        while($aCats = $oCats->fetchRow(MYSQLI_ASSOC)) {
                            $iQuestCounter = 0;
                            $iQuestCounter = $database->get_one('SELECT COUNT(*) FROM `'.TABLE_PREFIX.'mod_faq_questions` WHERE `section_id` = '.$iSectionId.' AND `cat_id` = '.$aCats['cat_id'].' ');
                            $aCat = array();
                            // items from current cat
                            $aCat['cat_id']          = $aCats['cat_id'];
                            $aCat['cat_idkey']       = $admin->getIDKEY($aCats['cat_id']);
                            $aCat['cat_desc']        = $aCats['cat_desc'];
                            $aCat['cat_name']        = $aCats['cat_name'];
                            $aCat['active']          = $aCats['active'];
                            $aCat['position']        = $aCats['position'];
                            $aCat['questCounter']    = $iQuestCounter;

                            $aResult[] = $aCat;
                        }
                    }
    //            echo "<pre>";
    //            print_r($aResult);
    //            echo "</pre>";
            // return the complete result structure
                return $aResult;
            }
        //
        ###############################################################################
        // backend overview - detailed view of a single category(getFAQCatDetailArray)
        ###############################################################################
        static public function getFAQCatDetailArray($iSectionId, $iCatId){ 
            global $database, $admin;
            // create an empty result
            $aResult = [];    
            // build the SQL-Statement to select from booth relating tables
            $sql = 'SELECT * FROM `'.TABLE_PREFIX.'mod_faq_categories` '
                                 . 'WHERE  `section_id` = '.$iSectionId.' '
                                 . 'AND `cat_id` = '.$iCatId.' ';
            if (($oCats = $database->query($sql))) {
                        while($aCats = $oCats->fetchRow(MYSQLI_ASSOC)) {
                        // items from current Cat
                        $aResult['cat_id']        = $aCats['cat_id'];
                        $aResult['cat_idkey']     = $admin->getIDKEY($aCats['cat_id']);
                        $aResult['cat_desc']      = $aCats['cat_desc'];
                        $aResult['cat_name']      = $aCats['cat_name'];
                        $aResult['active']        = $aCats['active'];
                        $aResult['position']      = $aCats['position'];
                        }
                    }
    //            echo "<pre>";
    //            print_r($aResult);
    //            echo "</pre>";
            // return the complete result structure
            return $aResult;
        } // end of function_exists('getFAQCatDetailArray')
        
        ###############################################################################
        // backend overview - detailed view of a single question(getFAQQuestDetailArray)
        ###############################################################################
        static public function getFAQQuestDetailArray($iSectionId, $iQuestionId){ 
                global $database, $admin,$sAddonName,$sAddonThemeUrl;
                // create an empty result
                $aResult = []; 


                // build the SQL-Statement to select from booth relating tables
                $sql = 'SELECT * FROM `'.TABLE_PREFIX.'mod_faq_questions` '
                     . 'WHERE  `section_id` = '.$iSectionId.' '
                     . 'AND `question_id` = '.$iQuestionId.' ';
                if (($oItem = $database->query($sql))) {
                        while($aItem = $oItem->fetchRow(MYSQLI_ASSOC)) {
                            if ($aItem['created_by'] == 0){
                                $sCreatedDisplayName = "unbekannt"; 
                            }else{ 
                                $sCreatedDisplayName  = $database->get_one('SELECT `display_name` FROM `'.TABLE_PREFIX.'users` WHERE `user_id` = '.(int)$aItem['created_by'].' ');
                            }
                            if ($aItem['modified_by'] == 0){
                                $sModifiedDisplayName = "unbekannt"; 
                            }else{ 
                                $sModifiedDisplayName = $database->get_one('SELECT `display_name` FROM `'.TABLE_PREFIX.'users` WHERE `user_id` = '.(int)$aItem['modified_by'].' ');
                            }
                            $sAnswerText = OutputFilterApi('ReplaceSysvar', $aItem['answer']);
                            $sAnswerText = htmlspecialchars_decode($sAnswerText);
                            // items from current group
                            $aResult['section_id']       = $aItem['section_id'];
                            $aResult['question_id']      = $aItem['question_id'];
                            $aResult['question_idkey']   = $admin->getIDKEY($aItem['question_id']);
                            $aResult['question']         = $aItem['question'];
                            $aResult['answer']           = $sAnswerText;
                            $aResult['keywords']         = $aItem['keywords'];
                            $aResult['cat_id']           = $aItem['cat_id'];
                            $aResult['active']           = $aItem['active'];
                            $aResult['position']         = $aItem['position'];
                            $aResult['published_when']   = $aItem['published_when'];
                            $aResult['published_until']  = $aItem['published_until'];
                            $aResult['modified_when']    = $aItem['modified_when'];
                            $aResult['modified_by']      = $aItem['modified_by'];
                            $aResult['modified_by_user'] = $sModifiedDisplayName;
                            $aResult['created_by_user']  = $sCreatedDisplayName;
                            $aResult['created_when']     = $aItem['created_when'];
                            $aResult['created_by']       = $aItem['created_by'];
                            $aResult['actual_time']      = time();
                        }
                    }
        //            echo "<pre>";
        //            print_r($aResult);
        //            echo "</pre>";
                // return the complete result structure
                return $aResult;
        } // end of function_exists('getFAQQuestDetailArray')
        //

        ###############################################################################
        // backend overview - detailed view of a single link(getFAQTemplateDetailArray)
        ###############################################################################
        static public function getFAQTemplateDetailArray($iSectionId, $iTemplateId){ 
                global $database, $admin,$sAddonName,$sAddonThemeUrl;
                // create an empty result
                $aResult = []; 
                // build the SQL-Statement to select from booth relating tables
                $sql = 'SELECT * FROM `'.TABLE_PREFIX.'mod_faq_templates` '
                     . 'WHERE  `section_id` = '.$iSectionId.' '
                     . 'AND `template_id` = '.$iTemplateId.' ';

                if (($oItem = $database->query($sql))) {
                        while($aItem = $oItem->fetchRow(MYSQLI_ASSOC)) {

                            // items from current group
                            $aResult['template_id']    = $aItem['template_id'];
                            $aResult['template_idkey'] = $admin->getIDKEY($aItem['template_id']);
                            $aResult['template_name']  = $aItem['template_name'];
                            $aResult['template_desc']  = $aItem['template_desc'];
                            $aResult['template_code']  = $aItem['template_code'];
                            $aResult['position']       = $aItem['position'];
                            $aResult['active']         = $aItem['active'];
                        }
                    }

                    #echo "<pre>";
                    #print_r($aResult);
                    #echo "</pre>";
                // return the complete result structure
                return $aResult;
        } // end of function_exists('getFAQTemplateDetailArray')




        ###############################################################################
        // frontend - read datas for frontend output(getFAQItemsViewArray) mit join
        ###############################################################################

        static public function getFAQItemsViewArray($iSectionId,$iCatOrder,$iQuestOrder){ 
            // import the global admin object
                global $database, $admin, $sAddonName,$sAddonTemplateUrl;
                if($iCatOrder == 1){ 
                    $iCatOrder = '  ORDER BY `c`.`position` ASC';
                }elseif($iCatOrder == 2){ 
                    $iCatOrder = '  ORDER BY `c`.`position` DESC';
                }elseif($iCatOrder == 3){ 
                    $iCatOrder = '  ORDER BY `c`.`cat_name` ASC';
                }else{ 
                    $iCatOrder = '  ORDER BY `c`.`cat_name` DESC';
                }
                if($iQuestOrder == '1'){ 
                    $iQuestOrder = '`q`.`position` ASC';
                }elseif($iQuestOrder == '2'){  
                    $iQuestOrder = '`q`.`position` DESC';
                }elseif($iQuestOrder == '3'){  
                    $iQuestOrder = '`q`.`question` ASC';
                }else{ 
                    $iQuestOrder = '`q`.`question` DESC';
                }

            // create an empty result
                $aResult = array();
                $iActualTime = time() + TIMEZONE;
            // initialize the Arrays for iteration
                $aCurrCat = $aCurrItem = null;
                if (!($iSectionId = intval($iSectionId))) {
                // return empty result if no possible sectionId is given
                    return $aResult;
                }

            // build the SQL-Statement to select from booth relating tables
                $sql = 'SELECT `c`.`cat_id` `cat_id`, `c`.`cat_name` `cat_name`, `c`.`section_id` `c_section_id`, '
                            . '`c`.`cat_desc` `cat_desc`, `c`.`active` `c_active`, '
                           .  '`q`.`question_id` `question_id`, `q`.`section_id` `q_section_id`, `q`.`active` `q_active`, '
                           .  '`q`.`question` `question`, `q`.`answer` `answer`, `q`.`keywords` `keywords`, '
                           .  '`q`.`cat_id` `q_cat_id`, `q`.`published_when`, `q`.`published_until`,  '
                           .  '`q`.`modified_when`, `q`.`modified_by`,  `q`.`created_when`, `q`.`created_by`  '
                     . 'FROM `'.TABLE_PREFIX.'mod_faq_questions` `q` '
                     . 'LEFT OUTER JOIN `'.TABLE_PREFIX.'mod_faq_categories` `c` ON `q`.`cat_id`=`c`.`cat_id` '
                     . 'WHERE `q`.`section_id`='.$iSectionId.' '
                     . ' AND `c`.`active`= 1 '
                     . ' AND `q`.`active`= 1 '
                     . 'AND ('
                     . '(`q`.`published_when` <= \''.$iActualTime .'\' OR `q`.`published_when` = 0) AND '
                     . '(`q`.`published_until` >= \''.$iActualTime .'\'  OR `q`.`published_until` = 0) '
                     . ') '
                     . ' '.$iCatOrder.', '.$iQuestOrder.'  ';
        //        echo "<br>".$sql;
                if (($oCatItems = $database->query($sql))) {
                // get the result object of the query
                    $iLastCatId = 0;
                    while($aCatItem = $oCatItems->fetchRow(MYSQLI_ASSOC)) {
                    // iterate over all matching records
                        if ($aCatItem['cat_id'] != $iLastCatId) {
                        // if a new group_id is detected then start a new group procceeding
                            if (!is_null($aCurrCat)) {
                            // if it's not the first run, first add the previous group to the result
                                $aResult[] = $aCurrCat;
                            }
                        // now create a new group
                            $aCurrCat = array();
                            $aCurrCat['cat_id']          = $aCatItem['cat_id'];
                            $aCurrCat['cat_idkey']       = $admin->getIDKEY($aCatItem['cat_id']);
                            $aCurrCat['cat_name']        = $aCatItem['cat_name'];
                            $aCurrCat['cat_desc']        = $aCatItem['cat_desc'];
                            $aCurrCat['cat_section']     = $aCatItem['c_section_id'];
                            $aCurrCat['active']          = $aCatItem['c_active'];
                            $aCurrCat['questions']       = array();
                        }
                        if (!is_null($aCatItem['question_id'])) {

                            // create a new item-array

                            $aCurrItem = array();
                            $aCurrItem['section_id']       = $aCatItem['q_section_id'];
                            $aCurrItem['active']           = $aCatItem['q_active'];
                            $aCurrItem['question_id']      = $aCatItem['question_id'];
                            $aCurrItem['question_idkey']   = $admin->getIDKEY($aCatItem['question_id']);
                            $aCurrItem['question']         = $aCatItem['question'];
                            $aCurrItem['answer']           = $aCatItem['answer'];
                            $aCurrItem['q_cat_id']         = $aCatItem['q_cat_id'];
                            $aCurrItem['published_when']   = $aCatItem['published_when'];
                            $aCurrItem['published_until']  = $aCatItem['published_until'];
                            $aCurrItem['modified_when']    = $aCatItem['modified_when'];
                            $aCurrItem['modified_by']      = $aCatItem['modified_by'];
                            $aCurrItem['created_when']     = $aCatItem['created_when'];
                            $aCurrItem['created_by']       = $aCatItem['created_by'];
                        // add this item to the current group
                            $aCurrCat['questions'][] = $aCurrItem;
                        }
                    // remember the last active group id
                        $iLastCatId = $aCatItem['cat_id'];
                    }
                }
                if (!is_null($aCurrCat)) {
                // add the last group to result if there is a group
                    $aResult[] = $aCurrCat;
                }

//                echo "<pre>";
//                print_r($aResult);
//                echo "</pre>";
            // return the complete result structure
                return $aResult;
        }// end of function_exists('getFAQItemsViewArray')
                
    } // end of class

// end of file
